function [X, measures, scan_vector] = parseLog(filename, MRI_scan_period, FS_Phys, nScans)
%
% [X, measures, scan_vector] = parseLog(filename, MRI_scan_period, FS_Phys, nScans)
%
% Assumes ECG-marker has value 1.
% X: column 1 = phase, 2 = HR, 3 = HRV, 4 = resp, 5 = resp-amp, 6 = resp-freq, 7 = resp_var, 8 = respPhase.
%
% Make sure FS_Phys is as exact as possible!
%
% Versie: 21:27, 22-07-2007.

% Variabelen
window_length = 6;
resp_window_length = 9;

% Code
% Tergend langzaam maar secure inlees code:
% file = fopen(filename, 'r');
% for n = 1:5, fgetl(file); end;
% d = [];
% linesread = 0;
% while ~feof(file),
%     l0 = fgetl(file);
%     linesread = linesread + 1;
%     if mod(linesread, 10000) == 0,
%         fprintf([num2str(linesread) '\n' num2str(d(end, :)) '\n']);
%     end;
%     v = str2num(l0);
%     if length(v) == 10,
%         d = [d; v];
%     end;
% end;
% fclose(file);
%
% Snelle inlees maar onzekerheid over regel op 7506:

[markervec, respvec] = parseLog_system_specific(filename);

markervec = cleanup_old(markervec, FS_Phys);
[measures.phase, measures.HR, measures.HRV, measures.resp, measures.respAmp, measures.respFreq, measures.RVT, measures.respPhase] = calculate(markervec, respvec, FS_Phys, window_length, resp_window_length, MRI_scan_period);
[X, scan_vector] = getPerScanVals(markervec, [measures.phase, measures.respPhase], [measures.HR, measures.HRV respvec(:), measures.respAmp, measures.respFreq, measures.RVT], MRI_scan_period, FS_Phys, nScans);
% replace outliers (esp. end-of-file zeroes) with means.
for col = 1:size(X, 2),
    z = find(X(:, col) == 0);
    nz = find(X(:, col) ~= 0);
    if length(nz) == 0,
        continue;
    end;
    X(z, col) = mean(X(nz, col));
end;

function m = cleanup_old(m, FS_Phys)
f = find(m == 1);
d = diff(f);
f2 = find(d < 0.5 * FS_Phys);
m(f(f2 + 1)) = 0;
f = find(m == 1);
d = diff(f);
f2 = find(d > 1.5 * FS_Phys);
fm = f(f2) + FS_Phys;
m(fm) = 1;

function X = cleanup_new(X, Fs)
f = find(X == 1);
d = diff(f);
m = mean(d);
sd = sqrt(var(d));
too_short = m - 3 * sd;
too_long = m + 3 * sd;
just_right = m;
OK = 0;
beenthere_short = 1;
beenthere_long = 1;
while OK == 0,
    OK = 1;
    
    f = find(X(beenthere_short:end) == 1);
    d = diff(f);
    fshort = find(d < too_short);
    s = [];
    if ~isempty(fshort),
        s = beenthere_short - 1 + f(fshort(1) + 1);
        X(s) = 0;
        beenthere_short = s;
        OK = 0;
    else,
        OK = 1;
    end;
    
    f = find(X(beenthere_long:end) == 1);
    d = diff(f);
    flong = find(d > too_long);
    if ~isempty(flong),
        base_marker = flong(1);
        n_to_add = ceil(d(base_marker) / just_right) - 1;
        interval = d(base_marker) / (n_to_add + 1);
        for m = 1:n_to_add,
            s = min(length(X), beenthere_long - 1 + f(base_marker) + floor(m * interval));
            X(s) = 1;
        end;
        beenthere_long = beenthere_long - 1 + f(base_marker + 1);
        OK = 0;
    else,
        OK = 1 * OK;
    end;
end;


function [phase, HR, HRV, respvec, respAmp, respFreq, RVT, respPhase] = calculate(markervec, respvec, FS_Phys, window_length, resp_window_length, MRI_scan_period)
skipper = round(MRI_scan_period * FS_Phys);
window0 = round(window_length * FS_Phys);
respwindow0 = round(resp_window_length * FS_Phys);
phase0 = find(markervec == 1);
if isempty(phase0),
    phase0 = find(markervec == 2);
end;
phase = zeros(size(markervec));
IBI = zeros(size(markervec));
HR = zeros(size(markervec));
HRV = zeros(size(markervec));
respAmp = zeros(size(markervec));
respFreq = zeros(size(markervec));
RVT = zeros(size(markervec));
respPhase = zeros(size(markervec));
for n = 2:length(phase0),
    a = phase0(n - 1);
    b = phase0(n);
    p0 = linspace(0, 2 * pi, b - a);
    phase(a:(b- 1)) = p0;
end;
currentBeat = 1;
for n = phase0(1):length(IBI),
    if n >= phase0(currentBeat + 1),
        currentBeat = currentBeat + 1;
        if currentBeat > length(phase0) - 1,
            break;
        end;
    end;
    IBI(n) = phase0(currentBeat + 1) - phase0(currentBeat);
end;
currentBeat = 1;
for n = phase0(1):skipper:length(HRV),
    a = max(1, n - ceil(window0/2));
    b = min(length(HRV), n + ceil(window0/2));
    r = IBI(a:b);
    r(find(r == 0)) = [];
    if mean(r) > 0,
        HR(n) = 1 / (mean(r) / FS_Phys);
    else,
        HR(n) = 0;
    end;
    HRV(n) = var(r / FS_Phys);
end;
f = find(HR > 0);
for iF = 2:length(f),
    a = f(iF - 1);
    b = f(iF);
    HR(a:b) = HR(a) + (HR(b) - HR(a)) * ((a:b) - a) / (b - a);
    HRV(a:b) = HRV(a) + (HRV(b) - HRV(a)) * ((a:b) - a) / (b - a);
end;
if var(respvec) ~= 0,
    [respAmp, respFreq] = twinpeaks(respvec, FS_Phys);
    RVT = respAmp .* respFreq;
    respPhase = resp_phase(respvec, FS_Phys, MRI_scan_period);
end;

function [X, scan_samples] = getPerScanVals(markervec, phase, preds, MRI_scan_period, FS_Phys, nScans_input)
X = [];
scan_samples = [];
scan0 = length(markervec);
scan0 = max(1, scan0 - floor(nScans_input * MRI_scan_period * FS_Phys));
% Visual check
vischeck = 0;
if vischeck == 1,
    r0 = max(1, scan0 - floor( 3* FS_Phys));
    r1 = min(length(markervec), scan0 + floor(3 * FS_Phys));
    figure(1); clf;
    subplot(2, 1, 2);
    plot(r0:r1, markervec(r0:r1), 'k-');
    hold on;
    startline0 = plot([scan0 scan0], [min(markervec(r0:r1)) max(markervec(r0:r1))], 'r-');
    xlim([r0 r1]);
    ylim([400 1300]);
end;
T = (length(markervec) - scan0 + 1) / FS_Phys;
nScans = min(nScans_input, floor(T / MRI_scan_period));
scan_samples = scan0 + floor((0:(nScans - 1)) * MRI_scan_period * FS_Phys);
scan_vector = zeros(size(markervec));
scan_vector(scan_samples) = max(markervec);
if vischeck == 1,
    subplot(2, 1, 1);
    plot(markervec, 'k-');
    hold on;
    y = max(markervec);
    startline1 = plot([scan_samples(1) scan_samples(1) ], [0 y], 'r-', 'Linewidth', 3);
    plot([scan_samples(end) scan_samples(end) ], [0 y], 'b-', 'Linewidth', 3);
end;
% predictors
for iScan = 1:length(scan_samples),
    a = floor(scan_samples(iScan));
    b = min(a + floor(MRI_scan_period * FS_Phys), length(markervec));
    X0 = mean(preds(a:b, :));
    X = [X; X0];
end;
% phase
p = [];
for iScan = 1:length(scan_samples),
    a = floor(scan_samples(iScan));
    p = [p; phase(a, :)];
end;
X = [p(:, 1) X p(:, 2)];

function [phase] = resp_phase(resp, Fs, MRI_scan_period);
phase = zeros(size(resp));
dRdt = zeros(size(resp));
%skipper = (MRI_scan_period * Fs) / 2;
skipper = 0.5 * Fs;
for n_frac = 1:skipper:length(resp),
    n = max(1, floor(n_frac));
    if n <= ceil(skipper / 2) || n >= floor(length(resp) - ceil(skipper / 2)),
        continue;
    end;
    %         fitopties = fitoptions('method', 'NonlinearLeastSquares', 'Lower', [0 0]);
    %         g = fittype({'1', 'x', 'x^2'});
    %         a = -floor(Fs / 2);
    %         b = +floor(Fs / 2);
    %         fit_params = fit((a:b)', resp(n + (a:b)), g);
    % Get dR/dt
    a = max(1, n - floor(skipper / 2));
    b = min(length(dRdt), n + floor(skipper / 2));
    [quadapprox, coeff] = thomas_quadapprox(a:b, resp(a:b));
    dRdt(a:b) = 2 * coeff(1) * ((a:b) - mean(a:b)) + coeff(2);
end;
[H, b] = hist(resp, 100);
total = length(resp);
for n = 1:length(resp),
    try,
        f = find(b <= resp(n));
        if isempty(f),
            temp = 0;
        else,
            temp = sum(H(f));
        end;
        phase(n) = pi * sign(dRdt(n)) * temp / total;
    catch,
        catcher0 = 0;
    end;
end;
catcher0 = 0;
